/*****************************************************************************/
/*                                                                   */
/*****************************************************************************/

#include "WWDisplayP.h"
#include "etc.h"

/*****************************************************************************/
/* дؿ                                                          */
/*****************************************************************************/

/*===========================================================================*/
/* Фμ                                                              */
/*===========================================================================*/

WWColorMap WWDisplay_GetColorMap(WWDisplay d) { return (d->color_map); }
WWPalette WWDisplay_GetPalette(WWDisplay d, int n) { return (d->palette[n]); }
WWCharacter WWDisplay_GetCharacter(WWDisplay d, int n)
{ return (d->character[n]); }
WWSprite WWDisplay_GetSprite(WWDisplay d, int n) { return (d->sprite[n]); }
WWScreen WWDisplay_GetScreen(WWDisplay d, int n) { return (d->screen[n]); }
WWLCDPanel WWDisplay_GetLCDPanel(WWDisplay d) { return (d->lcd_panel); }

int WWDisplay_GetSpriteEnable(WWDisplay d) { return (d->sprite_enable); }
int WWDisplay_GetSpriteWindowEnable(WWDisplay d)
{ return (d->sprite_window_enable); }

int WWDisplay_GetSpriteWindowX(WWDisplay d)
{ return (d->sprite_window_x); }
int WWDisplay_GetSpriteWindowY(WWDisplay d)
{ return (d->sprite_window_y); }
int WWDisplay_GetSpriteWindowWidth(WWDisplay d)
{ return (d->sprite_window_width); }
int WWDisplay_GetSpriteWindowHeight(WWDisplay d)
{ return (d->sprite_window_height); }

int WWDisplay_GetBorder(WWDisplay d) { return (d->border); }

int WWDisplay_GetForegroundColor(WWDisplay d) { return (d->foreground_color); }
int WWDisplay_GetBackgroundColor(WWDisplay d) { return (d->background_color); }

int WWDisplay_GetSpriteStart(WWDisplay d) { return (d->sprite_start); }
int WWDisplay_GetSpriteCount(WWDisplay d) { return (d->sprite_count); }

/*===========================================================================*/
/* Ф                                                              */
/*===========================================================================*/

WWColorMap WWDisplay_SetColorMap(WWDisplay d, WWColorMap cm)
{ return (d->color_map = cm); }
WWPalette WWDisplay_SetPalette(WWDisplay d, int n, WWPalette p)
{ return (d->palette[n] = p); }
WWCharacter WWDisplay_SetCharacter(WWDisplay d, int n, WWCharacter c)
{ return (d->character[n] = c); }
WWSprite WWDisplay_SetSprite(WWDisplay d, int n, WWSprite s)
{ return (d->sprite[n] = s); }
WWScreen WWDisplay_SetScreen(WWDisplay d, int n, WWScreen s)
{ return (d->screen[n] = s); }
WWLCDPanel WWDisplay_SetLCDPanel(WWDisplay d, WWLCDPanel p)
{ return (d->lcd_panel = p); }

int WWDisplay_SetSpriteEnable(WWDisplay d, int f)
{ return (d->sprite_enable = f); }
int WWDisplay_SetSpriteWindowEnable(WWDisplay d, int f)
{ return (d->sprite_window_enable = f); }

int WWDisplay_SetSpriteWindowX(WWDisplay d, int n)
{ return (d->sprite_window_x = n); }
int WWDisplay_SetSpriteWindowY(WWDisplay d, int n)
{ return (d->sprite_window_y = n); }
int WWDisplay_SetSpriteWindowWidth(WWDisplay d, int n)
{ return (d->sprite_window_width = n); }
int WWDisplay_SetSpriteWindowHeight(WWDisplay d, int n)
{ return (d->sprite_window_height = n); }

int WWDisplay_SetBorder(WWDisplay d, int b) { return (d->border = b); }

int WWDisplay_SetForegroundColor(WWDisplay d, int c)
{ return (d->foreground_color = c); }
int WWDisplay_SetBackgroundColor(WWDisplay d, int c)
{ return (d->background_color = c); }

int WWDisplay_SetSpriteStart(WWDisplay d, int n)
{ return (d->sprite_start = n); }
int WWDisplay_SetSpriteCount(WWDisplay d, int n)
{ return (d->sprite_count = n); }

/*===========================================================================*/
/* ֥ȤȾõ                                                  */
/*===========================================================================*/

WWDisplay WWDisplay_Create(int lcd_panel_width, int lcd_panel_height,
			   int screen_width, int screen_height)
{
  WWDisplay display;
  int i;

  display = (WWDisplay)malloc(sizeof(_WWDisplay));
  if (display == NULL)
    Wonx_Error("WWDisplay_Create", "Cannot allocate memory.");

  WWDisplay_SetColorMap(display, WWColorMap_Create(NULL));

  for (i = 0; i < 16; i++) {
    WWDisplay_SetPalette(display, i,
			 WWPalette_Create(i, NULL, ((i / 4) % 2) ? 1 : 0));
  }

  for (i = 0; i < 512; i++) {
    WWDisplay_SetCharacter(display, i, WWCharacter_Create(i, NULL));
  }

  for (i = 0; i < 128; i++) {
    WWDisplay_SetSprite(display, i,
			WWSprite_Create(i, 0, 0, 0, 0, 0, 0,
					WWDisplay_GetPalette(display, 0),
					WWDisplay_GetCharacter(display, 0)));
  }

  for (i = 0; i < 2; i++) {
    WWDisplay_SetScreen(display, i,
			WWScreen_Create(i, screen_width, screen_height,
					WWDisplay_GetPalette(display, 0),
					WWDisplay_GetCharacter(display, 0),
					0, 0,
					lcd_panel_width, lcd_panel_height));
  }

  WWDisplay_SetLCDPanel(display, WWLCDPanel_Create(lcd_panel_width,
						   lcd_panel_height));

  WWDisplay_SetSpriteEnable(display, 0);
  WWDisplay_SetSpriteWindowEnable(display, 0);

  WWDisplay_SetSpriteWindowX(     display, 0);
  WWDisplay_SetSpriteWindowY(     display, 0);
  WWDisplay_SetSpriteWindowWidth( display, lcd_panel_width);
  WWDisplay_SetSpriteWindowHeight(display, lcd_panel_height);

  WWDisplay_SetBorder(display, 0);

  WWDisplay_SetForegroundColor(display, 3);
  WWDisplay_SetBackgroundColor(display, 0);

  WWDisplay_SetSpriteStart(display, 0);
  WWDisplay_SetSpriteCount(display, 0);

  return (display);
}

WWDisplay WWDisplay_Destroy(WWDisplay display)
{
  int i;

  if (display == NULL) return (NULL);

  if (WWDisplay_GetColorMap(display) != NULL)
    WWDisplay_SetColorMap(display,
			  WWColorMap_Destroy(WWDisplay_GetColorMap(display)));

  for (i = 0; i < 16; i++) {
    if (WWDisplay_GetPalette(display, i) != NULL)
      WWDisplay_SetPalette(display, i,
			   WWPalette_Destroy(WWDisplay_GetPalette(display, i))
			   );
  }

  for (i = 0; i < 512; i++) {
    if (WWDisplay_GetCharacter(display, i) != NULL)
      WWDisplay_SetCharacter(display, i,
			     WWCharacter_Destroy(WWDisplay_GetCharacter(display
									, i)));
  }

  for (i = 0; i < 128; i++) {
    if (WWDisplay_GetSprite(display, i) != NULL)
      WWDisplay_SetSprite(display, i,
			  WWSprite_Destroy(WWDisplay_GetSprite(display, i)));
  }

  for (i = 0; i < 2; i++) {
    if (WWDisplay_GetScreen(display, i) != NULL)
      WWDisplay_SetScreen(display, i,
			  WWScreen_Destroy(WWDisplay_GetScreen(display, i)));
  }

  if (WWDisplay_GetLCDPanel(display) != NULL)
    WWDisplay_SetLCDPanel(display,
			  WWLCDPanel_Destroy(WWDisplay_GetLCDPanel(display)));

  free(display);

  return (NULL);
}

/*===========================================================================*/
/* LCDѥͥ                                                           */
/*===========================================================================*/

static int WWDisplay_DrawScreen(WWDisplay display, WWScreen screen)
{
  WWLCDPanel lcd_panel;
  int lcd_panel_width;
  int lcd_panel_height;

  int pixel;
  int x, y, px, py;
  int sx = 0, sy = 0, ex = 0, ey = 0;
  int mode;

  if (!WWScreen_GetEnable(screen)) return (0);

  lcd_panel = WWDisplay_GetLCDPanel(display);
  lcd_panel_width  = WWLCDPanel_GetWidth( lcd_panel);
  lcd_panel_height = WWLCDPanel_GetHeight(lcd_panel);

  if ( (WWScreen_GetMode(screen) == WW_SCREEN_INSIDE_ONLY) ||
       (WWScreen_GetMode(screen) == WW_SCREEN_OUTSIDE_ONLY) ) {
    sx = WWScreen_GetDrawX(screen);
    sy = WWScreen_GetDrawX(screen);

    /*
     * ޥ˥奢 screen2_set_window() ˤϡɽΰβȽפ
     * 񤤤Ƥ뤬ºݤˤϡɽΰβ+1Ƚ+1פư
     * ߤ(̤ǧ)ʤΤǡ-1 ʤƤ褤
     * ⤷ϡWonderWitch Ǥμºݤưϡ
     * ex = sx + WWScreen_GetDrawWidth( screen);
     * Ǥʤ
     * ex = WWScreen_GetDrawWidth( screen);
     * ʤΤ⤷ʤ
     */

#if 0
    ex = sx + WWScreen_GetDrawWidth( screen) - 1;
    ey = sy + WWScreen_GetDrawHeight(screen) - 1;
#else
    ex = sx + WWScreen_GetDrawWidth( screen);
    ey = sy + WWScreen_GetDrawHeight(screen);
#endif
  }

  mode = WWScreen_GetMode(screen);

  /* ʲϥۥåȥݥåȤˤʤΤǡΤ롼ץ */
  /* ۤ                                           */

  for (y = 0; y < lcd_panel_height; y++) {

    if (mode == WW_SCREEN_INSIDE_ONLY) {
      if (y > ey) { break; }
      if (y < sy) { y = sy - 1; continue; }
    }

    py = y + WWScreen_GetRollY(screen);

    for (x = 0; x < lcd_panel_width; x++) {

      if (mode == WW_SCREEN_INSIDE_ONLY) {
	if (x > ex) { x = lcd_panel_width - 1; continue; }
	if (x < sx) { x = sx - 1; continue; }
      } else if (mode == WW_SCREEN_OUTSIDE_ONLY) {
	if ( (x >= sx) && (x <= ex) && (y >= sy) && (y <= ey) ) {
	  x = ex;
	  continue;
	}
      }

      px = x + WWScreen_GetRollX(screen);

      pixel = WWScreen_GetPixel(screen, px, py);

      /* Ʃξ */
      if (pixel == -1) continue;

      pixel = WWColorMap_GetLCDColor(WWDisplay_GetColorMap(display), pixel);
      WWLCDPanel_SetPixel(lcd_panel, x, y, pixel);
    }
  }

  return (0);
}

static int WWDisplay_DrawSprite(WWDisplay display, WWSprite sprite)
{
  int x, y, lcd_x, lcd_y;
  int sx, sy, ex, ey;
  int pixel, outside;

  sx = WWDisplay_GetSpriteWindowX(display) - 1;
  sy = WWDisplay_GetSpriteWindowY(display) - 1;

  /* ʲ WWDisplay_DrawScreen() ˹碌 */
#if 0
  ex = sx + WWDisplay_GetSpriteWindowWidth(display) - 1;
  ey = sy + WWDisplay_GetSpriteWindowHeight(display) - 1;
#else
  ex = sx + WWDisplay_GetSpriteWindowWidth(display);
  ey = sy + WWDisplay_GetSpriteWindowHeight(display);
#endif

  for (y = 0; y < 8; y++) {
    for (x = 0; x < 8; x++) {
      pixel = WWSprite_GetPixel(sprite, x, y);

      /* Ʃξ */
      if (pixel == -1) continue;

      lcd_x = WWSprite_GetX(sprite) + x;
      lcd_y = WWSprite_GetY(sprite) + y;

      if (WWDisplay_GetSpriteWindowEnable(display)) {
	if ( (lcd_x < sx) || (lcd_y < sy) || (lcd_x > ex) || (lcd_y > ey) )
	  outside = 1;
	else
	  outside = 0;

	if (WWSprite_GetClipping(sprite)) { /* ɥγ¦ʬɽ */
	  if (!outside) continue;
	} else {                            /* ɥ¦ʬɽ */
	  if (outside) continue;
	}
      }

      pixel = WWColorMap_GetLCDColor(WWDisplay_GetColorMap(display), pixel);
      WWLCDPanel_SetPixel(WWDisplay_GetLCDPanel(display), lcd_x, lcd_y, pixel);
    }
  }

  return (0);
}

int WWDisplay_DrawLCDPanel(WWDisplay display)
{
  WWLCDPanel lcd_panel;
  int x, y, i;
  int lcd_panel_width;
  int lcd_panel_height;
  WWColorMap color_map;
  int border;
  WWSprite sprite;

  lcd_panel = WWDisplay_GetLCDPanel(display);
  lcd_panel_width  = WWLCDPanel_GetWidth( lcd_panel);
  lcd_panel_height = WWLCDPanel_GetHeight(lcd_panel);
  color_map = WWDisplay_GetColorMap(display);
  border = WWDisplay_GetBorder(display);

  /* ܡ顼 */
  for (x = 0; x < lcd_panel_width; x++) {
    for (y = 0; y < lcd_panel_height; y++) {
      WWLCDPanel_SetPixel(lcd_panel, x, y,
			  WWColorMap_GetLCDColor(color_map, border));
    }
  }

  /* ꡼ */
  WWDisplay_DrawScreen(display, WWDisplay_GetScreen(display, 0));

  /* ץ饤(꡼󣲤ͥǤʤ) */
  /* Ťʤäϡֹμ㤤Τɽ */
  if (WWDisplay_GetSpriteEnable(display)) {
    for (i = WWDisplay_GetSpriteCount(display) - 1; i >= 0; i--) {
      sprite = WWDisplay_GetSprite(display,
				   i + WWDisplay_GetSpriteStart(display));
      if (!WWSprite_GetPriority(sprite)) {
	WWDisplay_DrawSprite(display, sprite);
      }
    }
  }

  /* ꡼ */
  WWDisplay_DrawScreen(display, WWDisplay_GetScreen(display, 1));

  /* ץ饤(꡼󣲤ͥʤ) */
  /* Ťʤäϡֹμ㤤Τɽ */
  if (WWDisplay_GetSpriteEnable(display)) {
    for (i = WWDisplay_GetSpriteCount(display) - 1; i >= 0; i--) {
      sprite = WWDisplay_GetSprite(display,
				   i + WWDisplay_GetSpriteStart(display));
      if (WWSprite_GetPriority(sprite)) {
	WWDisplay_DrawSprite(display, sprite);
      }
    }
  }

  return (0);
}

/*****************************************************************************/
/* ޤ                                                                  */
/*****************************************************************************/

/*****************************************************************************/
/* End of File.                                                              */
/*****************************************************************************/
